/**
 * Driver for Kraken
 **/
var RPX_REG = /[-+]?\d*\.?\d+(rpx)/g;
var NON_DIMENSIONAL_REG = /opa|ntw|ne[ch]|ex(?:s|g|n|p|$)|^ord|zoo|grid|orp|ows|mnc|^columns$|bs|erim|onit/i;
var EVENT_PREFIX_REG = /^on[A-Z]/;
var DANGEROUSLY_SET_INNER_HTML = 'dangerouslySetInnerHTML';
var CLASS_NAME = 'className';
var CLASS = 'class';
var STYLE = 'style';
var CHILDREN = 'children';
var TEXT_CONTENT_ATTR = 'textContent';
var CREATE_ELEMENT = 'createElement';
var CREATE_COMMENT = 'createComment';
var CREATE_TEXT_NODE = 'createTextNode';
var SET_ATTRIBUTE = 'setAttribute';
var REMOVE_ATTRIBUTE = 'removeAttribute';
var EMPTY = '';
var viewportWidth = 750;
var unitPrecision = 4;
/**
 * Create a cached version of a pure function.
 */

function cached(fn) {
  var cache = Object.create(null);
  return function cachedFn(str) {
    return cache[str] || (cache[str] = fn(str));
  };
}

var isEventProp = cached(function (prop) {
  return EVENT_PREFIX_REG.test(prop);
});
function createBody() {
  return document.body;
}
function createEmpty(component) {
  return document[CREATE_COMMENT](EMPTY);
}
function createText(text, component) {
  return document[CREATE_TEXT_NODE](text);
}
function updateText(node, text) {
  node[TEXT_CONTENT_ATTR] = text;
}
function createElement(type, props, component, __shouldConvertUnitlessToRpx, __shouldConvertRpxToVw) {
  if (__shouldConvertRpxToVw === void 0) {
    __shouldConvertRpxToVw = true;
  }

  var node = document[CREATE_ELEMENT](type);

  for (var prop in props) {
    var value = props[prop];
    if (prop === CHILDREN) continue;

    if (value != null) {
      if (prop === STYLE) {
        setStyle(node, value, __shouldConvertUnitlessToRpx, __shouldConvertRpxToVw);
      } else if (isEventProp(prop)) {
        addEventListener(node, prop.slice(2).toLowerCase(), value);
      } else {
        setAttribute(node, prop, value);
      }
    }
  }

  return node;
}
function appendChild(node, parent) {
  return parent.appendChild(node);
}
function removeChild(node, parent) {
  parent = parent || node.parentNode; // Maybe has been removed when remove child

  if (parent) {
    parent.removeChild(node);
  }
}
function replaceChild(newChild, oldChild, parent) {
  parent = parent || oldChild.parentNode;
  parent.replaceChild(newChild, oldChild);
}
function insertAfter(node, after, parent) {
  parent = parent || after.parentNode;
  var nextSibling = after.nextSibling;

  if (nextSibling) {
    // Performance improve when node has been existed before nextSibling
    if (nextSibling !== node) {
      insertBefore(node, nextSibling, parent);
    }
  } else {
    appendChild(node, parent);
  }
}
function insertBefore(node, before, parent) {
  parent = parent || before.parentNode;
  parent.insertBefore(node, before);
}
function addEventListener(node, eventName, eventHandler) {
  return node.addEventListener(eventName, eventHandler);
}
function removeEventListener(node, eventName, eventHandler) {
  return node.removeEventListener(eventName, eventHandler);
}
function removeAttribute(node, propKey) {
  if (propKey === DANGEROUSLY_SET_INNER_HTML) return;
  if (propKey === CLASS_NAME) propKey = CLASS;

  if (propKey in node) {
    try {
      // Some node property is readonly when in strict mode
      node[propKey] = null;
    } catch (e) {}
  }

  node[REMOVE_ATTRIBUTE](propKey);
}
function setAttribute(node, propKey, propValue) {
  // For reduce innerHTML operation to improve performance.
  if (propKey === DANGEROUSLY_SET_INNER_HTML) {
    warnUnsupport(DANGEROUSLY_SET_INNER_HTML);
    return;
  }

  if (propKey === CLASS_NAME) propKey = CLASS;

  if (propKey in node) {
    try {
      // Some node property is readonly when in strict mode
      node[propKey] = propValue;
    } catch (e) {
      node[SET_ATTRIBUTE](propKey, propValue);
    }
  } else {
    node[SET_ATTRIBUTE](propKey, propValue);
  }
}

function toFixed(number, precision) {
  var multiplier = Math.pow(10, precision + 1);
  var wholeNumber = Math.floor(number * multiplier);
  return Math.round(wholeNumber / 10) * 10 / multiplier;
}

function unitTransformer(n) {
  return toFixed(parseFloat(n) / (viewportWidth / 100), unitPrecision) + 'vw';
}

function isRpx(str) {
  return RPX_REG.test(str);
}

function calcRpxToVw(value) {
  return value.replace(RPX_REG, unitTransformer);
}

var isDimensionalProp = cached(function (prop) {
  return !NON_DIMENSIONAL_REG.test(prop);
});
var convertUnit = cached(function (value) {
  return isRpx(value) ? calcRpxToVw(value) : value;
});
function setStyle(node, style, __shouldConvertUnitlessToRpx, __shouldConvertRpxToVw) {
  if (__shouldConvertRpxToVw === void 0) {
    __shouldConvertRpxToVw = true;
  }

  for (var prop in style) {
    var value = style[prop];
    var convertedValue = void 0;

    if (typeof value === 'number' && isDimensionalProp(prop)) {
      if (__shouldConvertUnitlessToRpx) {
        convertedValue = value + 'rpx';

        if (__shouldConvertRpxToVw) {
          // Transfrom rpx to vw
          convertedValue = convertUnit(convertedValue);
        }
      } else {
        convertedValue = value + 'px';
      }
    } else {
      convertedValue = __shouldConvertRpxToVw ? convertUnit(value) : value;
    } // Support CSS custom properties (variables) like { --main-color: "black" }


    if (prop[0] === '-' && prop[1] === '-') {
      // reference: https://developer.mozilla.org/en-US/docs/Web/API/CSSStyleDeclaration/setProperty.
      // style.setProperty do not support Camel-Case style properties.
      node.style.setProperty(prop, convertedValue);
    } else {
      node.style[prop] = convertedValue;
    }
  }
}

function warnUnsupport(message) {
  console.warn("[DriverKraken]: " + message + " is not supported.");
}

export { addEventListener, appendChild, createBody, createElement, createEmpty, createText, insertAfter, insertBefore, removeAttribute, removeChild, removeEventListener, replaceChild, setAttribute, setStyle, updateText };
//# sourceMappingURL=driver-kraken.mjs.map
